--
--  Linkmate: Quests
--    by Tuhljin
--

local TjQuestLog = TjQuestLog


local ADDON_QUESTGURU = IsAddOnLoaded("QuestGuru")


-- REPLACE "QUEST ACCEPTED" CHAT
----------------------------------

local questMsgFrames, acceptedQuests = {}

local function MultiframeAddMessage(frames, ...)
	for frame in pairs(frames) do
		frame:AddMessage(...)
	end
end

local function OnQuestLogChange()
	if (not acceptedQuests) then  return;  end
	local info = ChatTypeInfo["SYSTEM"]
	local log = TjQuestLog.GetDataTable(true)
	for id,tab in pairs(log) do
		if (acceptedQuests[tab.questTitle]) then
			local msg = GetQuestLink(tab.index)
			msg = ERR_QUEST_ACCEPTED_S:format(msg)
			MultiframeAddMessage(questMsgFrames, msg, info.r, info.g, info.b, info.id)
			acceptedQuests[tab.questTitle] = nil
		end
	end
	TjQuestLog.ResetHeaderStates()
	for name in pairs(acceptedQuests) do
	-- If a quest somehow wasn't found, print a bare "accepted" message:
		local msg = ERR_QUEST_ACCEPTED_S:format(name)
		--DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id);
		MultiframeAddMessage(questMsgFrames, msg, info.r, info.g, info.b, info.id)
	end
	wipe(questMsgFrames)
	acceptedQuests = nil
end
TjQuestLog.Register_OnChange(OnQuestLogChange)

local SearchQuestAccepted = gsub(ERR_QUEST_ACCEPTED_S, "%%s", "(.+)")  -- Get quest name from "Quest accepted: %s"

local function filter(self, event, arg1, ...)
	local _, _, name = strfind(arg1, SearchQuestAccepted);
	if (name) then
	-- The quest log is not updated at the time we receive this message so the message is filtered out for now:
		acceptedQuests = acceptedQuests or {}
		acceptedQuests[name] = true
		questMsgFrames[self] = true
		return true;
	end
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)


-- HYPERLINK HOOK
-------------------

--local orig_SetItemRef = SetItemRef
local orig_ChatFrame_OnHyperlinkShow = ChatFrame_OnHyperlinkShow
ChatFrame_OnHyperlinkShow = function(self, link, text, button, ...)
	if (strsub(link, 1, 6) == "quest:") then
		if (IsControlKeyDown()) then
			local _, id = strsplit(":", link)
			id = tonumber(id)
			local q = TjQuestLog.GetDataTable(true)[id]
			if (q) then
				QuestLog_OpenToQuest(q.index)
				TjQuestLog.ResetHeaderStates()
				local i, isCollapsed = TjQuestLog.GetHeaderInfo(q.header)
				if (i and isCollapsed) then  ExpandQuestHeader(i);  end
				if (ADDON_QUESTGURU) then  QuestLogFrame:Show(); end
			else
				TjQuestLog.ResetHeaderStates()
			end
			return;
		end
	end
	return orig_ChatFrame_OnHyperlinkShow(self, link, text, button, ...)
end
